<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Base;
use App\Http\Controllers\Controller;
use App\Http\Requests\AddCategoryRequest;
use App\Http\Requests\DeleteCategoryRequest;
use App\Http\Requests\ToggleCategoryRequest;
use App\Http\Requests\UpdateCategoryRequest;
use App\Services\Interface\CategoryInterface;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

class CategoryController extends Controller
{
    //
    private CategoryInterface $categoryRepository;

    public function __construct(CategoryInterface $categoryRepository)
    {
        $this->categoryRepository = $categoryRepository;
    }

    public function index()
    {
        try {
            $data = $this->categoryRepository->index();
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function add(AddCategoryRequest $request){
        try{
            $data = $this->categoryRepository->add($request);
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        }
        catch(Exception $e){
            return Base::exception($e);
        }
    }

    public function update(UpdateCategoryRequest $request){
        try {
            $data = $this->categoryRepository->update($request);
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }
    public function toggle(ToggleCategoryRequest $request){
        try {
            $data = $this->categoryRepository->toggle($request);
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }
    public function delete(DeleteCategoryRequest $request){
        try {
            $data = $this->categoryRepository->delete($request);
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function addRecommended(Request $request)
    {
        try{
            $validator =  Validator::make($request->all(), [
                'category_id' => 'required',
            ]);
            if ($validator->fails()) return Base::validation($validator);
           $data = $this->categoryRepository->addRecommended($request);
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        }
        catch(Exception $e){
            return Base::exception($e);
        }
    }
    public function getRecommendedCategoryList()
    {
        try{
            $data = $this->categoryRepository->getRecommendedCategoryList();
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        }
        catch(Exception $e){
            return Base::exception($e);
        }
    }

}
