<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Base;
use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\Order;
use App\Models\PostCard;
use App\Models\Shop;
use App\Models\User;
use Carbon\Carbon;
use Exception;
use Illuminate\Support\Facades\DB;

class DashboardController extends Controller
{
    //! as dashboard need just one function, so I haven't make any repository for this one

    public function getDashboardData()
    {
        try {

            
            $recent_orders = $this->getRecentOrders();

            $top_selling_cards_of_month = $this->getTopSellingCardsOfMonth();
            $analytics = $this->getAnalytics();

            $data = [
                'total_card' => PostCard::count(),
                'total_shop' => Shop::count(),
                'total_category' => Category::count(),
                'total_customer' => User::where('role', 'customer')->count(),
                'recent_orders' => $recent_orders,
                'top_selling_cards_of_month' => $top_selling_cards_of_month,
                'analytics' => $analytics,
            ];

            return Base::success('Dashboard Information', $data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
            return Base::error($e->getMessage());
        }
    }

    private function getRecentOrders()
    {
        $data = [];
        $orders = Order::orderBy('created_at', 'desc')->take(5)->get();

        foreach ($orders as $order) {
            $required_order_data = [
                'id' => $order->id,
                'card_name' => $order->postCard->name ?? '',
                'card_image' => $order->postCard->image ?? '',
                'quantity' => $order->quantity ?? 0,
                'price' => $order->price ?? 0,
                'payment_method' => $order->payment_method ?? '',
                'customer_name' => $order->user->name ?? '',
                'customer_phone' => $order->user->phone ?? '',
                'address' => $order->ship_address ?? '',
            ];
            array_push($data, $required_order_data);
        }

        return $data;
    }

    private function getTopSellingCardsOfMonth()
    {

        //! TODO: Need to refactor this function, this is temporary solution
        $data = [];
        $card = [];

        $month = Carbon::now()->month;

        $orders = Order::whereMonth('created_at', $month)->with('card')->get(['card_id']);
        $item2 = [];

        foreach($orders as $item)
        {
            $order_count = Order::whereMonth('created_at', $month)->where('card_id',$item->card_id)->where('status', 'completed')->count();
            $order_total = Order::whereMonth('created_at', $month)->where('card_id',$item->card_id)->where('status', 'completed')->get();
            $total_price = 0;
            foreach($order_total as $k){
                $total_price = $total_price + $k->order_total;
            }
            $item2 = [
                "card_id" => $item->card_id,
                "card_name" => $item->card_name,
                "card_image" => $item->card_image,
                "card_price" => $item->card->price,
                "card_total_order" => $order_count,
                "card_total_sale" => $total_price,
            ];
            array_push($data, $item2);

            
        }

        $data = collect($data)->unique('card_id')->sortByDesc('card_total_sale')->take(5)->values()->all();



        return $data;
    }

    private function getAnalytics()
    {
        $data = [];
        $month = Carbon::now()->month;

        $total_sale_order = Order::whereMonth('created_at', $month)->where('status','completed')->count();
        $total_pending_order = Order::whereMonth('created_at', $month)->whereIn('status',['pending','processing'])->count();
        $total_cancel_order = Order::whereMonth('created_at', $month)->whereIn('status',['canceled', 'rejected'])->count();

        if ($total_sale_order != 0 || $total_pending_order != 0 || $total_cancel_order != 0) {
            $s_total_order = (100 / ($total_sale_order + $total_pending_order + $total_cancel_order)) * $total_sale_order;
            $p_total_order = (100 / ($total_sale_order + $total_pending_order + $total_cancel_order)) * $total_pending_order;
            $c_total_order = (100 / ($total_sale_order + $total_pending_order + $total_cancel_order)) * $total_cancel_order;
        } else {
            $s_total_order = $total_sale_order;
            $p_total_order = $total_pending_order;
            $c_total_order = $total_cancel_order;
        }

        return $data = [
            "total_sale_order" => ($s_total_order),
            "total_pending_order" => ($p_total_order),
            "total_cancel_order" => ($c_total_order),
        ];
    }
}
