<?php

namespace App\Http\Controllers;

use App\Models\Contact;
use App\Services\Interface\ContactInterface;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class ContactController extends Controller
{
    public function index(ContactInterface $i_contact, Request $request) {
        try {
            $contacts = $i_contact->index(Auth::id());
            if(!$contacts->success) return Base::error($contacts->message);
            
            return Base::success('Contact list fetched', $contacts->data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }
    
    public function add(ContactInterface $i_contact, Request $request) {
        try {
            $validator =  Validator::make($request->all(), [
                'name' => 'required',
                'email' => 'required',
                'phone' => 'required',
                'postcode' => 'required',
                'address' => 'required',
            ]);
            if ($validator->fails()) return Base::validation($validator);

            $inputs = $request->all();
            
            $inputs['user_id'] = Auth::id();

            $res = $i_contact->add($inputs);
            if(!$res->success) return Base::error($res->message);

            return Base::success('Contact Info Added', $res->data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function update(ContactInterface $i_contact, Request $request) {
        try {
            $validator =  Validator::make($request->all(), [
                'id' => 'required',
            ]);
            if ($validator->fails()) return Base::validation($validator);

            $contact = Contact::find($request->id);
            if(!$contact) return Base::error("Contact not found.");

            $res = $i_contact->update($contact, $request->all());
            if(!$res->success) return Base::error($res->message);

            return Base::success('Contact Updated', $res->data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function delete(ContactInterface $i_contact, Request $request) {
        try {
            $validator =  Validator::make($request->all(), [
                'id' => 'required',
            ]);
            if ($validator->fails()) return Base::validation($validator);

            $contact = Contact::find($request->id);
            if(!$contact) return Base::error("Contact not found"); // DONE

            $res = $i_contact->delete($contact);
            if(!$res->success) return Base::error($res->message);

            return Base::success('Contact Deleted', $res->data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }
}
