<?php

namespace App\Http\Controllers\Customer;

use App\Http\Controllers\Base;
use App\Http\Controllers\Controller;
use App\Http\Requests\AddAddressBookRequest;
use App\Http\Requests\UpdateAddressBookRequest;
use App\Services\Interface\AddressBookInterface;
use Exception;
use Illuminate\Http\Request;

class AddressBookController extends Controller
{
    //
    private AddressBookInterface $addressBookRepository;

    public function __construct(AddressBookInterface $addressBookRepository)
    {
        $this->addressBookRepository = $addressBookRepository;
    }

    public function index(){
        try {
            $data = $this->addressBookRepository->index();
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function add(AddAddressBookRequest $request){
        try{
            $data = $this->addressBookRepository->add($request);
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        }
        catch(Exception $e){
            return Base::exception($e);
        }
    }
    public function update(UpdateAddressBookRequest $request){
        try{
            $data = $this->addressBookRepository->update($request);
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        }
        catch(Exception $e){
            return Base::exception($e);
        }
    }

    public function delete(UpdateAddressBookRequest $request)
    {
        try {
            $data = $this->addressBookRepository->delete($request);
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }
}
