<?php

namespace App\Http\Controllers;

use App\Models\AppNotification;
use App\Models\Notification;
use App\Models\User;
use App\Services\Interface\NotificationInterface;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class NotificationController extends Controller
{
    //
    private NotificationInterface $notificationRepository;

    public function __construct(NotificationInterface $notificationRepository)
    {
        $this->notificationRepository = $notificationRepository;
    }

    public function index(Request $request)
    {
        try {
            return $this->notificationRepository->index($request);
        } catch (Exception $e) {
            Base::exception($e);
        }
    }

    public function setDeviceToken(Request $request)
    {

        try {
            $validator = Validator::make($request->all(), [
                'user_id' => 'required',
                'device_token' => 'required',
            ]);
            if ($validator->fails()) {
                return Base::error('Validation Error', $validator->errors()); // DONE
            }
            $user = Auth::user();

            if ($user->id != $request->user_id) {
                return Base::error('User_id and Auth user_id does not match'); // DONE
            }

            $user_app = AppNotification::where(['device_token' => $request->device_token])->first();

            if ($user_app != null) {
                if ($user_app->user_id == $request->user_id) {
                    if ($user_app->subscription == false) {
                        $user_app->subscription = true;
                        $user_app->save();
                        return Base::success('User Subscription enabled for this app'); // DONE
                    } else {
                        return Base::success('User Subscription already enabled for this app'); // DONE
                    }
                } else {
                    $user_app->delete();

                    $new_user_app = new AppNotification();
                    $new_user_app->user_id = $request->user_id;
                    $new_user_app->device_token = $request->device_token;
                    $new_user_app->subscription = true;
                    $new_user_app->save();

                    return Base::success('New App added for user'); // DONE
                }
            } else {
                $new_user_app = new AppNotification();
                $new_user_app->user_id = $request->user_id;
                $new_user_app->device_token = $request->device_token;
                $new_user_app->subscription = true;
                $new_user_app->save();
                return Base::success('New App added for user'); // DONE
            }
        } catch (\Exception $e) {
            return Base::error('An error occurred', $e->getMessage()); // DONE
        }
    }

    public function setNotification(Request $request)
    {
        $data = [
            "to_id" => $request->to_id,
            "from_id" => $request->from_id,
            "type" => $request->type,
            "title_key" => $request->title_key,
            "description_key" => $request->description_key,
        ];
        $to_id = $data['to_id'];
        $from_id = $data['from_id'];
        $type = $data['from_id'];
        $title_key = $data["title_key"];
        $description_key = $data['description_key'];

        $user = User::find($to_id);

        if ($user != null) {
            $new_notification = new Notification();
            $new_notification->to_id = $to_id;
            $new_notification->from_id = $from_id;
            $new_notification->type = $type;
            $new_notification->title_key = $title_key;
            $new_notification->description_key = $description_key;
            $new_notification->save();

            $notification = array(
                "title" => $title_key,
                "body" => $description_key
            );

            if ($user->is_push_notification == true) {
                $notification = [
                    'user' => $to_id,
                    'notification' => $notification
                ];
                $noti = AppNotification::pushNotification($notification);
            }
            if ($user->is_email_notification == true) {
                ///TODO
            }

            $data = $new_notification;

            return Base::success('New notification has been save', $data); // DONE

        } else {
            return Base::error('User not found'); // DONE
        }
    }

    public function show(Request $request)
    {
        try {
            return $this->notificationRepository->show($request);
        } catch (Exception $e) {
            Base::exception($e);
        }
    }

    public function markAllUnread(Request $request)
    {
        try {
            $user_id = auth()->user()->id;
            // $ids = $request->ids;
            $notification = Notification::where('to_id', $user_id)->update(['is_seen' => false]);
            return Base::success('All notification marked as unread', $notification); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }
    public function markAllRead(Request $request)
    {
        try {
            $user_id = auth()->user()->id;
            // $ids = $request->ids;
            $notification = Notification::where('to_id', $user_id)->update(['is_seen' => true]);
            return Base::success('All notification marked as read', $notification); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }
}
