<?php

namespace App\Http\Requests;

use App\Http\Controllers\Base;
use Illuminate\Contracts\Validation\Validator;
use Illuminate\Foundation\Http\FormRequest;
use Illuminate\Http\Exceptions\HttpResponseException;

class AdminAddCustomerRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array<string, mixed>
     */
    public function rules()
    {
        return [
            'name' => 'required|string|min:4',
            'email' => 'required|string|email|unique:users',
            'role' => 'required|string|in:customer,shop',
            'password' => 'required|confirmed|min:6',
        ];
    }

    public function failedValidation(Validator $validator)
    {
        throw new HttpResponseException(Base::validation($validator));
        // return Base::validation($validator);
    }

    public function messages()
    {
        return [
            "name.required" => "Name is required",
            "email.required" => "Email required",
            "role.required" => "Role is required",
            "role.in" => "Role type is invalid"
            // "name.unique" => "This category already exist"
        ];
    }
}
