<?php

namespace App\Models;

use App\Http\Controllers\Base;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Order extends Model
{
    use HasFactory;
    protected $appends = ['order_date', 'card_name', 'card_image', 'order_total'];
    protected $casts = [
        'id' => 'integer',
        'price' => 'double',
        'quantity' => 'integer',
        'authority_fee' => 'integer',
        'shipping_fee' => 'integer',
        'discount' => 'integer',
        'uer_id' => 'integer',
    ];

    public function postCard()
    {
        return $this->hasOne(PostCard::class, 'id', 'card_id');
    }
    public function card()
    {
        return $this->hasOne(PostCard::class, 'id', 'card_id');
    }
    public function postCardWithShop()
    {
        return $this->hasOne(PostCard::class, 'id', 'card_id')->with('shop');
    }

    public function orderPayment()
    {
        return $this->hasOne(OrderPayment::class, 'order_id')->latest();
    }

    public function getOrderDateAttribute()
    {
        return Base::carbonParse($this->created_at);
    }

    public function getCardNameAttribute()
    {
        return $this->postCard->name;
    }

    public function getCardImageAttribute()
    {
        return $this->postCard->image;
    }

    public function getOrderTotalAttribute()
    {
        $this->unsetRelation('postCard');
        $total = (($this->price * $this->quantity) + $this->shipping_fee + $this->authority_fee) - $this->discount;
        return $total;
    }

    public function user()
    {
        return $this->belongsTo(User::class, 'user_id');
    }
    public function customer()
    {
        return $this->belongsTo(User::class, 'user_id');
    }
}
