<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Shop extends Model
{
    use HasFactory, SoftDeletes;

    protected $hidden = [
        'deleted_at',
        'created_at',
        'updated_at',
    ];

    protected $casts = [
        'user_id' => 'integer',
    ];


    public function User()
    {
        return $this->belongsTo(User::class);
    }

    public function userData()
    {
        return $this->belongsTo(User::class, 'user_id')->select('id', 'name', 'email', 'phone', 'address', 'image', 'image_url', 'is_verified', 'is_active', 'created_at');
    }

    public function postCards()
    {
        return $this->hasMany(PostCard::class, 'shop_id', 'user_id')
            ->join('categories', 'post_cards.category_id', '=', 'categories.id')
            // ->join('shops', 'post_cards.shop_id', '=', 'shops.id')
            ->join('users', 'post_cards.shop_id', '=', 'users.id')
            ->select("post_cards.name", 'users.is_active as user_is_active', 'categories.is_active as category_is_active', 'post_cards.*')
            ->where([
                ['categories.is_active', '=', 1],
                ['users.is_active', '=', 1],
            ]);
    }

    public function postCards2()
    {
        return $this->hasMany(PostCard::class, 'shop_id', 'user_id')
            ->join('categories', 'post_cards.category_id', '=', 'categories.id')
            // ->join('shops', 'post_cards.shop_id', '=', 'shops.id')
            ->join('users', 'post_cards.shop_id', '=', 'users.id')
            ->select("post_cards.name", 'users.is_active as user_is_active', 'categories.is_active as category_is_active', 'post_cards.*');
    }

    public function orders()
    {
        return $this->hasMany(Order::class, 'user_id');
    }

    public function pendingOrders()
    {
        return $this->hasMany(Order::class, 'user_id')->where('status', 'pending');
    }

    public function completedOrders()
    {
        return $this->hasMany(Order::class, 'user_id')->where('status', 'completed');
    }
}
