<?php

namespace App\Providers;

use App\Services\Interface\ContactInterface;
use App\Services\Interface\ContactUsInterface;
use App\Services\Interface\GeneralInterface;
use App\Services\Interface\OrderInterface;
use App\Services\Interface\UserInterface;
use App\Services\Interface\PostCardInterface;
use App\Services\Interface\ShopInterface;
use App\Services\Interface\TestInterface;
use App\Services\Repositories\ContactRepository;
use App\Services\Repositories\ContactUsRepository;
use App\Services\Repositories\GeneralRepository;
use App\Services\Repositories\OrderRepository;
use App\Services\Repositories\UserRepository;
use App\Services\Repositories\PostCardRepository;
use App\Services\Repositories\ShopRepository;
use App\Services\Repositories\TestRepository;
use Illuminate\Support\ServiceProvider;
use Illuminate\Support\Collection;
use Illuminate\Pagination\LengthAwarePaginator;

class AppServiceProvider extends ServiceProvider
{
    /**
     * Register any application services.
     *
     * @return void
     */
    public function register()
    {
        //* registering repository service provider to the Main app service providers

        $this->app->register(RepositoryServiceProvider::class);
        $this->app->bind(GeneralInterface::class, GeneralRepository::class);
    }

    /**
     * Bootstrap any application services.
     *
     * @return void
     */
    public function boot()
    {
        /**
         * Paginate a standard Laravel Collection.
         *
         * @param int $perPage
         * @param int $total
         * @param int $page
         * @param string $pageName
         * @return array
         */
        Collection::macro('paginate', function($perPage, $total = null, $page = null, $pageName = 'page') {
            $page = $page ?: LengthAwarePaginator::resolveCurrentPage($pageName);

            return new LengthAwarePaginator(
                $this->forPage($page, $perPage),
                $total ?: $this->count(),
                $perPage,
                $page,
                [
                    'path' => LengthAwarePaginator::resolveCurrentPath(),
                    'pageName' => $pageName,
                ]
            );
        });
    }
}
