<?php

namespace App\Services\Repositories;

use App\Http\Controllers\Base;
use App\Http\Resources\OrderResource;
use App\Models\Feedback;
use App\Models\Order;
use App\Models\OrderPayment;
use App\Models\PostCard;
use App\Services\Interface\OrderInterface;

use App\Models\User;
use App\Models\Voucher;
use App\Traits\GlobalTrait;
use Carbon\Carbon;
use Exception;
use Illuminate\Support\Facades\Auth;

class OrderRepository implements OrderInterface
{
    use GlobalTrait;

    public function add($data)
    {
        try {
            if (isset($data['order_id'])) {
                $order = Order::find($data['order_id']);
                if ($order == null) return Base::fail("Order is not found"); // DONE
                $check_payment = OrderPayment::where("payment_id", $data['payment_id'])->first();

                if ($check_payment != null) $check_payment->delete();
            } else $order = new Order();

            $check_payment = $this->checkPayment($data['payment_id']);
            if ($check_payment->success == false) return Base::fail($check_payment->message);
            $payment_status = $check_payment->data;

            $code = $this->generateCode();

            $order->user_id = $data['user_id'];
            $order->card_id = $data['card_id'];

            $order->code = "OID-" . $code;
            $order->quantity = $data['quantity'] ?? 1;
            // TODO:: text_image upload
            $order->text_image = $data['text_image'] ? GlobalTrait::orderImageUpload($data['text_image'], 'Order Image', "OID-" . $code) : null;
            $order->text = $data['text'] ?? null;
            $order->message = $data['message'] ?? null;
            $order->price = $data['price'];
            $order->authority_fee = $data['authority_fee']; //! TODO: get from backend
            $order->shipping_fee = $data['shipping_fee']; //! TODO: get from backend
            $order->discount = $data['discount'] ?? 0;

            // $order->status = 'pending';

            $order->bill_name = $data['bill_name'];
            // $order->bill_email = $data['bill_email'];
            $order->bill_phone = $data['bill_phone'];
            $order->bill_postcode = $data['bill_postcode'];
            $order->bill_city = $data['bill_city'];
            $order->bill_address = $data['bill_address'];
            $order->bill_address_type = $data['bill_address_type'];

            // $order->is_save_for_signup = $data['is_save_for_signup'] ?? false;

            $order->ship_name = $data['ship_name'];
            // $order->ship_email = $data['ship_email'];
            $order->ship_phone = $data['ship_phone'];
            $order->ship_postcode = $data['ship_postcode'];
            $order->ship_city = $data['ship_city'];
            $order->ship_address = $data['ship_address'];
            $order->ship_address_type = $data['ship_address_type'];

            // $order->delivery_type = $data['delivery_type'];
            $order->delivery_date = isset($data['delivery_date']) ? Carbon::parse($data['delivery_date']) : null;
            $order->delivery_time = isset($data['delivery_time']) ? Carbon::parse($data['delivery_time']) : null;

            $order->payment_method = $data['payment_method'];
            // $order->expiry = Carbon::parse($data['expiry']) ?? null;
            // $order->is_save_card = $data['is_save_card'] ?? false;

            $order->discount_code = $data['discount_code'] ?? null;
            $order->is_gift_paper = $data['is_gift_paper'] ?? false;

            $order->status = $payment_status == "payment_due" ? "payment_due" : "pending";

            $order->save();
            if (isset($data['discount_code']) && $data['discount_code'] != null) {
                $this->voucherApply($data['discount_code']);
            }

            $order->refresh();
            $payment_data = [
                "payment_id" => $data['payment_id'],
                "order_id" => $order->id,
                "status" => $payment_status,
            ];
            // $payment_data->order_id = $order->id;
            $payment_save = $this->savePayment($payment_data);
            // $payment_save;
            if ($payment_save->success == false) {
                $order->delete();
                if (file_exists($order->text_image)) {
                    @unlink($order->text_image);
                }
                return Base::fail("Something went wrong"); // DONE
            }
            $order_data = [
                "order" => $order,
                "payment_data" => $payment_save->data,
            ];

            return Base::pass(null, $order_data);
        } catch (Exception $e) {
            return $e;
            return Base::exception_fail($e);
        }
    }

    public function checkPayment($payment_id)
    {
        try {
            $secret_key = config('app.stripe_key');

            $stripe = new \Stripe\StripeClient($secret_key);
            $data =   $stripe->paymentIntents->retrieve(
                $payment_id,
                []
            );

            if ($data->status == 'requires_payment_method') {
                $status = "payment_due";
            } else if ($data->status == 'succeeded') {
                $status = "succeeded";
            } else return Base::fail("Payment Verification failed"); // DONE

            $check_payment = OrderPayment::where("payment_id", $data->id)->first();
            if ($check_payment != null) return Base::fail("Duplicate Payment Id");

            return Base::pass("Payment verified", $status); // DONE

        } catch (Exception $e) {
            // return $e;
            return Base::exception_fail($e);
        }
    }

    public function savePayment($payment)
    {
        try {

            $new_payment = new OrderPayment();
            $new_payment->order_id = $payment['order_id'];
            $new_payment->status = $payment['status'];
            $new_payment->payment_id = $payment['payment_id'];
            $new_payment->save();
            $data = $new_payment->refresh();

            return Base::pass("Payment added", $data); // DONE

        } catch (Exception $e) {
            // return $e;
            return Base::exception($e);
        }
    }

    public function confirmPayment($data)
    {
        try {
            $order_id = $data['order_id'];
            $payment_id = $data['payment_id'];

            $order = Order::find($order_id);
            if (!$order) return Base::fail("Order not found"); // DONE
            if ($order->status != 'payment_due') return Base::fail("Order is not in payment status"); // DONE
            $order_payment = OrderPayment::where("payment_id", $payment_id)->first();
            if (!$order_payment) return Base::fail("Payment information not found"); // DONE

            if ($order_id != $order_payment->order_id) return Base::fail("Payment is not initialized in this order"); // DONE

            $secret_key = config('app.stripe_key');

            $stripe = new \Stripe\StripeClient($secret_key);
            $payment_data =   $stripe->paymentIntents->retrieve(
                $order_payment->payment_id,
                []
            );
            // $payment_data = $payment_data;
            if ($payment_data->status == 'succeeded') {
                $status = "succeeded";
            } else return  Base::fail("Payment Verification failed"); // DONE



            $order->status = "pending";

            $order_payment->status = $status;

            $order_payment->type = $payment_data->payment_method_types[0];
            $order_payment->amount = $payment_data->amount;
            $order_payment->currency = $payment_data->currency;
            $order_payment->payment_method_id = $payment_data->charges->data[0]->payment_method;
            $order_payment->card_brand = $payment_data->charges->data[0]->payment_method_details->card->brand;
            $order_payment->card_country =  $payment_data->charges->data[0]->payment_method_details->card->country;
            $order_payment->card_exp_month =  $payment_data->charges->data[0]->payment_method_details->card->exp_month;
            $order_payment->card_exp_year =  $payment_data->charges->data[0]->payment_method_details->card->exp_year;
            $order_payment->card_4digit =  $payment_data->charges->data[0]->payment_method_details->card->last4;
            $order_payment->save();
            $order->save();

            return Base::pass("Payment successfully"); // DONE


        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function addFeedback($order, $rate, $review)
    {
        try {
            $order->rate = $rate;
            $order->review = $review;
            $order->save();

            return Base::pass(null, $order);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function orderHistory($user_id)
    {
        try {
            $orders = Order::where('user_id', $user_id)->with('postCard')->get();

            return Base::pass(null, $orders);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function orderHistoryShow($data)
    {
        try {
            $order = Order::where('id', $data['id'])->with(['postCardWithShop', 'customer', 'orderPayment'])->latest()->first();

            $secret_key = config('app.stripe_key');

            $status = null;

            if (isset($data['payment_id'])) {
                $payment_id = $data['payment_id'];
                $stripe = new \Stripe\StripeClient($secret_key);
                $payment_details =   $stripe->paymentIntents->retrieve(
                    $payment_id,
                    []
                );

                if ($payment_details->status == 'requires_payment_method' && $order->status == "payment_due") {
                    $status = "payment_due";
                } else if ($payment_details->status == 'succeeded' && $order->status == "payment_due") {
                    $status = "checkout_due";
                }
            }
            $order->payment_status = $status;


            $order->order_date = Base::carbonDateParse($order->created_at);
            if (!$order) return Base::fail("Order not found"); // DONE

            return Base::pass(null, $order);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function generateCode()
    {
        $str_result = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
        $random = substr(str_shuffle($str_result), 0, 6);

        $found_duplicate = Order::where('code', $random)->first();
        if ($found_duplicate) $this->generateCode();

        return $random;
    }

    public function shopOrderIndex()
    {
        try {
            $user = Auth::user();
            $orders = Order::with('postCard')
                ->whereHas('postCard', function ($q) use ($user) {
                    $q->where('shop_id', $user->id);
                })
                ->orderByDesc('created_at')
                ->paginate(20);

            return Base::pass("Shop order list", $orders); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function orderAction($data)
    {
        try {

            $order = Order::find($data->id);
            if (!$order) return Base::fail('Order not found'); // DONE

            $user = Auth::user();
            if ($user->role != 'admin' || $user->role != 'sa') {
                if ($user->role == 'customer' && $order->user_id != $user->id) return Base::fail("Unauthorized user"); // DONE

                if ($user->role == 'shop' && $order->postCard->shop_id != $user->id) return Base::fail("Unauthorized user" . $order->postCard->shop_id,); // DONE
            }

            $action  = null;

            if (($data->action == 'processing' || $data->action == 'canceled' || $data->action == 'rejected') && $order->status == 'pending') $action = $data->action;
            if (($data->action == 'completed' || $data->action == 'canceled' || $data->action == 'rejected') && $order->status == 'processing') $action = $data->action;
            // if ($order->status == 'completed' || $order->status == 'rejectd')
            if ($action == null) return Base::fail("Can not take action on this stage"); // DONE

            $order->status = $action;
            $order->save();

            return Base::pass("Order status has been updated"); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function orderHistoryFilter($data)
    {
        try {
            //code...
            $user = Auth::user();
            //! keep it for now, check this later
            // if ($user->role == 'sa' || $user->role == 'admin') 
            // {
            //     if (isset($request->customer_id)) $request['customer_id'] = $request->customer_id;
            //     if (isset($request->shop_id)) $request['shop_id'] = $request->shop_id;
            //     if (isset($request->shop_id) && isset($request->customer_id)) {
            //         $request['customer_id'] = $request->customer_id;
            //         $request['shop_id'] = $request->shop_id;
            //     }
            // }
            if ($user->role == 'customer') $data['customer_id'] = $user->id;
            if ($user->role == 'shop') $data['shop_id'] = $user->id;

            $take = isset($data['take']) ? $data['take'] : 20;
            $shop_id = isset($data['shop_id']) ? $data['shop_id'] : null;
            $customer_id = isset($data['customer_id']) ? $data['customer_id'] : null;

            $min_date = isset($data['min_date']) ? Base::carbonParse($data['min_date']) : null;
            $max_date = isset($data['max_date']) ? Base::carbonParse($data['max_date']) : null;
            $status = isset($data['status']) ? $data['status'] : null;
            $card_id = isset($data['card_id']) ? $data['card_id'] : null;
            $search = isset($data['search']) ? $data['search'] : null;

            if (isset($data['status'])) {
                $status = $data['status'] == "ongoing" ? ['pending', 'processing'] : [$data['status']];
            }
            if ($customer_id == null && $shop_id == null) {
                $orders = Order::with('postCardWithShop', 'customer');
            }
            if ($shop_id != null) {
                $orders = Order::with('postCardWithShop', 'customer')->with('postCard')
                    ->whereHas('postCard', function ($q) use ($shop_id) {
                        $q->where('shop_id', $shop_id);
                    });
            }

            if ($card_id != null) {
                $orders = Order::where('card_id', $card_id)->with('postCardWithShop', 'customer');
            }

            if ($customer_id != null) {
                $orders = Order::where('user_id', $customer_id)->with('postCardWithShop', 'customer');
            }
            if ($customer_id != null && $shop_id != null) {
                $orders = Order::where('user_id', $customer_id)
                    ->with('postCard')
                    ->whereHas('postCard', function ($q) use ($shop_id) {
                        $q->where('shop_id', $shop_id);
                    });
            }
            $orders = $orders->when($search != null, function ($q) use ($search) {
                $q->where('code', 'like', '%' . $search . '%');
            })
                ->when($status != null, function ($q) use ($status) {

                    $q->whereIn('status', $status);
                })
                ->when($min_date != null, function ($q) use ($min_date) {
                    $q->where('created_at', '>=', $min_date);
                })
                ->when($max_date != null, function ($q) use ($max_date) {
                    $q->where('created_at', '<=', $max_date);
                })
                ->with('orderPayment')
                ->orderBy('created_at', 'desc')
                ->paginate($take);

            return Base::pass("Orders filtered list", $orders); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function homePendingOrder($data)
    {
        try {
            $customer_id = $data['customer_id'];
            $orders = Order::where('user_id', $customer_id)
                ->whereIn('status', ['pending', 'processing'])
                ->select('id', 'card_id', 'code', 'quantity', 'status', 'price', 'authority_fee', 'shipping_fee', 'discount')
                ->with('orderPayment')
                ->orderBy('created_at', 'desc')
                ->take($data['take'])->get();
            $data = $orders;

            return Base::pass(null, $data);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function voucherApply($data)
    {
        try {
            $voucher_code = $data;
            $voucher = Voucher::where('voucher_code', $voucher_code)->first();
            if (!$voucher) {
                $msg = "Voucher is not exist.";
                $value = false;
                return Base::fail($msg, $value);
            }
            $voucher->no_used = $voucher->no_used + 1;
            if (($voucher->no_used + 1) == $voucher->limit) {
                $voucher->status = 'used';
            }
            $voucher->save();
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }
}
