<?php

namespace App\Services\Repositories;

use App\Http\Controllers\Base;
use App\Models\Category;
use App\Models\PostCard;
use App\Models\RecommendedCategory;
use App\Models\Shop;
use App\Services\Interface\PostCardInterface;

use App\Models\User;
use App\Services\Interface\CategoryInterface;
use App\Traits\GlobalTrait;
use Carbon\Carbon;
use Exception;
use Illuminate\Support\Facades\Auth;

class PostCardRepository implements PostCardInterface
{

    use GlobalTrait;

    public function getCategories()
    {
        try {
            $categories = Category::active()->select('id', 'name', 'image')->get();

            return Base::pass(null, $categories);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function filter($shop_id = null, $request = null)
    {
        try {
            // $post_cards = PostCard::where('shop_id', $shop_id)->get();
            // return $request;

            $role = Auth::user()->role ?? null;

            $category = isset($request->categories) ? $request->categories : [];
            $search = isset($request->search) ? $request->search : null;
            $min_price = isset($request->min_price) ? $request->min_price : null;
            $max_price = isset($request->max_price) ? $request->max_price : null;
            $type = isset($request->type) ? $request->type : null;

            $take = isset($request->take) && $request->take != null ? $request->take : 20;

            $shop_id = isset($request->shop_id) ? $request->shop_id : null;

            if ($shop_id !== null) {
                // return 123;
                $post_cards = PostCard::join('categories', 'post_cards.category_id', '=', 'categories.id')
                    // ->join('shops', 'post_cards.shop_id', '=', 'shops.id')
                    ->join('users', 'post_cards.shop_id', '=', 'users.id')
                    ->select("post_cards.name", 'users.is_active as user_is_active', 'categories.is_active as category_is_active', 'post_cards.*')
                    ->when( $role == null || !in_array($role, ['sa', 'admin']), function ($q) {
                        $q->where([
                            ['categories.is_active', '=', 1],
                            ['users.is_active', '=', 1],
                        ]);
                    })
                    ->where('shop_id', $shop_id)
                    ->when($search != null, function ($query) use ($search) {
                        return $query->where('post_cards.name', 'like', '%' . $search . '%');
                    })
                    ->when($min_price != null, function ($query) use ($min_price) {
                        return $query->where('price', '>=', $min_price);
                    })
                    ->when($max_price != null, function ($query) use ($max_price) {
                        return $query->where('price', '<=', $max_price);
                    })
                    ->when($category != null, function ($query) use ($category) {
                        return $query->whereIn('category_id', $category);
                    })
                    ->withCount('sold')
                    ->orderByDesc('rate')
                    ->paginate($take)
                    ->withQueryString();
                $post_cards->appends(['shop_id' => $shop_id]);
            } else {
                 $post_cards = PostCard::join('categories', 'post_cards.category_id', '=', 'categories.id')
                    // ->join('shops', 'post_cards.shop_id', '=', 'shops.id')
                    ->join('users', 'post_cards.shop_id', '=', 'users.id')
                    ->select("post_cards.name", 'users.is_active as user_is_active', 'categories.is_active as category_is_active', 'post_cards.*')
                    ->when($role == null || !in_array($role, ['sa', 'admin']), function ($q) {
                        $q->where([
                            ['categories.is_active', '=', 1],
                            ['users.is_active', '=', 1],
                        ]);
                    })
                    ->when($search, function ($query) use ($search) {
                        return $query->where('post_cards.name', 'like', '%' . $search . '%');
                    })
                    ->when($category, function ($query) use ($category) {
                        return $query->whereIn('category_id', $category);
                    })
                    ->when($min_price, function ($query) use ($min_price) {
                        return $query->where('price', '>=', $min_price);
                    })
                    ->when($max_price, function ($query) use ($max_price) {
                        return $query->where('price', '<=', $max_price);
                    })
                    ->orderByDesc('rate')
                    ->paginate($take);


                // $post_cards = PostCard::when($search, function ($query) use ($search) {
                //     return $query->where('name', 'like', '%' . $search . '%');
                // })
                //     ->when($category, function ($query) use ($category) {
                //         return $query->whereIn('category_id', $category);
                //     })
                //     ->when($min_price, function ($query) use ($min_price) {
                //         return $query->where('price', '>=', $min_price);
                //     })
                //     ->when($max_price, function ($query) use ($max_price) {
                //         return $query->where('price', '<=', $max_price);
                //     })
                //     // ->withCount('sold')
                //     ->orderByDesc('rate')
                //     ->paginate($take);
            }

            if ($type == 'popular_cards') {
                $post_cards = PostCard::join('categories', 'post_cards.category_id', '=', 'categories.id')
                    ->join('shops', 'post_cards.shop_id', '=', 'shops.id')
                    ->join('users', 'post_cards.shop_id', '=', 'users.id')
                    ->select("post_cards.name", 'users.is_active as user_is_active', 'categories.is_active as category_is_active', 'post_cards.*')
                    ->where([
                        ['categories.is_active', '=', 1],
                        ['users.is_active', '=', 1],
                    ])->when($search, function ($query) use ($search) {
                        return $query->where('name', 'like', '%' . $search . '%');
                    })
                    ->when($category, function ($query) use ($category) {
                        return $query->whereIn('category_id', $category);
                    })
                    ->when($min_price, function ($query) use ($min_price) {
                        return $query->where('price', '>=', $min_price);
                    })
                    ->when($max_price, function ($query) use ($max_price) {
                        return $query->where('price', '<=', $max_price);
                    })

                    ->withCount('sold')->orderBy('sold_count', 'desc')
                    ->paginate(20);
            }
            if ($type == 'new_cards') {
                $post_cards = PostCard::where('created_at', '>=', Carbon::now()->subDay())
                    ->when($search, function ($query) use ($search) {
                        return $query->where('name', 'like', '%' . $search . '%');
                    })
                    ->when($category, function ($query) use ($category) {
                        return $query->whereIn('category_id', $category);
                    })
                    ->when($min_price, function ($query) use ($min_price) {
                        return $query->where('price', '>=', $min_price);
                    })
                    ->when($max_price, function ($query) use ($max_price) {
                        return $query->where('price', '<=', $max_price);
                    })
                    ->withCount('sold')->orderBy('sold_count', 'desc')
                    ->paginate($take);
            }

            return Base::pass("Post card list fetched", $post_cards);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function popularIndex($limit = null)
    {
        try {
            // $post_cards = PostCard::orderBy('rate', 'ASC')->limit($limit)->get();
            $post_cards = PostCard::join('categories', 'post_cards.category_id', '=', 'categories.id')
                // ->join('shops', 'post_cards.shop_id', '=', 'shops.id')
                ->join('users', 'post_cards.shop_id', '=', 'users.id')
                ->select("post_cards.name", 'users.is_active as user_is_active', 'categories.is_active as category_is_active', 'post_cards.*')
                ->where([
                    ['categories.is_active', '=', 1],
                    ['users.is_active', '=', 1],
                ])
                ->withCount('sold')->orderBy('sold_count', 'desc')->limit($limit)->get();

            return Base::pass(null, $post_cards);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }
    public function recommendedIndex($limit = 6)
    {
        try {

            $category = [];
            $get_recommended_category_data =
                RecommendedCategory::where('is_active', true)->take(10)->get();
            foreach ($get_recommended_category_data as $item) {
                array_push($category, $item['category_id']);
            }
            $post_cards = PostCard::join('categories', 'post_cards.category_id', '=', 'categories.id')
                ->join('shops', 'post_cards.shop_id', '=', 'shops.id')
                ->join('users', 'post_cards.shop_id', '=', 'users.id')
                ->select("post_cards.name", 'users.is_active as user_is_active', 'categories.is_active as category_is_active', 'post_cards.*')
                ->where([
                    ['categories.is_active', '=', 1],
                    ['users.is_active', '=', 1],
                ])
                ->whereIn('category_id', $category)->orderBy('rate', 'ASC')->limit($limit)->get();

            return Base::pass("Recommended card lists", $post_cards); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function show($id)
    {
        try {
            // $post_card = PostCard::find($id);
            $post_card = PostCard::where('id', $id)->with('category')->first();
            if (!$post_card) return Base::fail("Post Card not found"); // DONE

            $related_card = PostCard::where('category_id', $post_card->category_id)->inRandomOrder()
                ->limit(3) // here is yours limit
                ->get();
            $data = [
                "post_card" => $post_card,
                "related_card" => $related_card->except([$id]),
            ];

            return Base::pass("Post card details fetch", $data); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function adminPostCardShow($data)
    {
        try {
            //Code will be here
            $post_card = PostCard::where('id', $data['card_id'])->with(['category', 'shop'])->first();
            if (!$post_card) return Base::fail("Post Card not found"); // DONE

            return Base::pass("Post Card details show", $post_card); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function add($data)
    {
        try {
            $post_card = new PostCard();
            $post_card->shop_id = $data['shop_id'];
            $post_card->category_id = $data['category_id'];
            $post_card->name = $data['name'];
            $post_card->code = $this->generateCode();
            $post_card->price = $data['price'];

            if (isset($data['image'])) {
                $post_card->image = GlobalTrait::imageUpload($data['image'], 'post_cards');
            }
            $post_card->save();

            return Base::pass(null, $post_card);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function generateCode()
    {
        $str_result = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
        $random = substr(str_shuffle($str_result), 0, 6);

        $found_duplicate = PostCard::where('code', $random)->first();
        if ($found_duplicate) $this->generateCode();

        return $random;
    }

    public function update($post_card, $data)
    {
        try {
            // return $data;
            $post_card->category_id = $data['category_id'] ?? $post_card->category_id;
            $post_card->name = $data['name'] ?? $post_card->name;
            // $post_card->code = $data['code'] ?? $post_card->code;
            $post_card->price = $data['price'] ?? $post_card->price;
            $post_card->shop_id = $data['shop_id'] ?? $post_card->shop_id;


            if (isset($data['image'])) {
                if ($post_card->image) {
                    if (is_file(public_path() . '/' . $post_card->image)) {
                        unlink(public_path() . '/' . $post_card->image);
                    }
                }
                $post_card->image = GlobalTrait::imageUpload($data['image'], 'post_cards');
            }
            $post_card->save();

            return Base::pass(null, $post_card);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function delete($post_card)
    {
        try {
            // TODO: Unlink Image
            if ($post_card->image) {
                if (is_file(public_path() . '/' . $post_card->image)) {
                    unlink(public_path() . '/' . $post_card->image);
                }
            }
            $post_card->delete();

            return Base::pass(null, $post_card);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function statusToggle($request)
    {
        try {
            $post_card = PostCard::find($request['id']);
            if ($post_card == null) return Base::fail("Card not found"); // DONE
            $post_card->is_active = $request->is_active ? true : false;

            $post_card->save();

            return Base::pass("Post Card status toggled successfully", $post_card); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function updateRate($card_id)
    {
        try {
            $post_card = PostCard::find($card_id);
            if (!$post_card) return Base::fail("Post card not found"); // DONE

            $ratedOrders = $post_card->ratedOrders;
            $rate_sum = $ratedOrders->sum('rate');
            $total_feedbacks = $ratedOrders->count();

            $overall_rate = number_format((float)($rate_sum / $total_feedbacks), 1, '.', '');
            $post_card->rate = $overall_rate;
            $post_card->save();

            return Base::pass(null, $post_card);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }
}
