<?php

use App\Http\Controllers\Admin\CategoryController;
use App\Http\Controllers\Admin\CustomerController;
use App\Http\Controllers\Admin\DashboardController;
use App\Http\Controllers\Admin\FAQController;
use App\Http\Controllers\Admin\PostCardController;
use App\Http\Controllers\Admin\SettingsController;
use App\Http\Controllers\Admin\TermsPrivacyController;
use App\Http\Controllers\Admin\VoucherController;
use App\Http\Controllers\BoostedShopController;
use App\Http\Controllers\OrderController;
use App\Http\Controllers\PostCardController as ControllersPostCardController;
use App\Http\Controllers\ProfileController;
use App\Http\Controllers\ShopController;
use App\Http\Controllers\TestController;
use Illuminate\Support\Facades\Route;



Route::get('testa', [TestController::class, 'index']);

Route::get('/dashboard', [DashboardController::class, 'getDashboardData']);

Route::group(['prefix' => 'category'], function () {

    Route::get('index', [CategoryController::class, 'index']);
    Route::post('add', [CategoryController::class, 'add']);
    Route::post('update', [CategoryController::class, 'update']);
    Route::post('toggle', [CategoryController::class, 'toggle']);
    Route::post('delete', [CategoryController::class, 'delete']);


    Route::post('update-recommended-category', [CategoryController::class, 'addRecommended']);
    //! missing api
    // customer details 
});

Route::group(['prefix' => 'customer'], function(){
    Route::get('index', [CustomerController::class, 'index']);
    Route::get('filter', [CustomerController::class, 'adminCustomerFilter']);
    Route::get('show', [CustomerController::class, 'show']);
    Route::post('update', [CustomerController::class, 'update']);
    Route::post('add', [CustomerController::class, 'addCustomer']);
    Route::post('change-password', [CustomerController::class, 'adminCustomerChangePassword']);
    Route::post('delete', [CustomerController::class, 'adminDeleteCustomer']);
    Route::post('status-toggle', [CustomerController::class, 'statusToggle']);
    Route::post('vid-status-toggle', [CustomerController::class, 'vidStatusToggle']);

    Route::group(['prefix' => 'order'], function(){
        Route::post('filter', [OrderController::class, 'orderHistoryFilter']);
    });
});

// todo:: work on shop api
Route::group(['prefix' => 'shop'], function () {

    Route::get('get-all-shops', [ShopController::class, 'getAllShops']);
    Route::get('get-shops-filter', [ShopController::class, 'getShopFilter']);
    Route::post('add-new-shop', [ShopController::class, 'addNewShop']);
    Route::post('update-shop', [ShopController::class, 'updateShop']);
    Route::post('delete-shop', [ShopController::class, 'deleteShop']);
    Route::post('show-shop', [ShopController::class, 'showShop']);
    Route::post('cards', [ShopController::class, 'shopCards']);
    Route::post('order/filter', [OrderController::class, 'orderHistoryFilter']);
    // todo:: shop details, status toggle and ... api
    
});

//! gift-cards, orders and settings api missing

Route::group(['prefix' => 'orders'], function(){
    Route::post('filter', [OrderController::class, 'orderHistoryFilter']);
    Route::post('show', [OrderController::class, 'orderHistoryShow']);
    Route::post('action', [OrderController::class, 'orderAction']);
});

Route::group(['prefix' => 'post-cards'], function (){
    Route::any('filter', [PostCardController::class, 'filter']);
    Route::get('show', [PostCardController::class, 'show']);
    Route::post('order-filter', [OrderController::class, 'orderHistoryFilter']);
    Route::post('add', [ControllersPostCardController::class, 'add']);
    Route::post('update', [ControllersPostCardController::class, 'update']);
    Route::post('delete', [ControllersPostCardController::class, 'delete']);
    Route::post('status-toggle', [ControllersPostCardController::class, 'statusToggle']);
});



Route::group(['prefix' => 'voucher'], function () {
    Route::get('index', [VoucherController::class, 'index']);
    Route::post('status-toggle', [VoucherController::class, 'voucherToggle']);
    Route::post('delete', [VoucherController::class, 'voucherDelete']);
    Route::post('add', [VoucherController::class, 'add']);
    Route::post('details', [VoucherController::class, 'voucherDetails']);
    Route::post('filter', [VoucherController::class, 'voucherFilter']);

});

Route::group(['prefix' => 'faq'], function (){
    Route::get('index', [FAQController::class, 'index']);
    Route::post('add', [FAQController::class, 'add']);
    Route::post('update', [FAQController::class, 'update']);
    Route::post('delete', [FAQController::class, 'delete']);
    Route::post('status-toggle', [FAQController::class, 'statusToggle']);

});

Route::group(['prefix' => 'terms-and-condition'], function(){
    Route::get('get', [TermsPrivacyController::class, 'getTC']);
    Route::post('update', [TermsPrivacyController::class, 'addTCPrivacy']);
});
Route::group(['prefix' => 'privacy-policy'], function(){
    Route::get('get', [TermsPrivacyController::class, 'getPrivacy']);
    Route::post('update', [TermsPrivacyController::class, 'addTCPrivacy']);
});
Route::group(['prefix' => 'settings'], function(){
    Route::get('get', [SettingsController::class, 'getFees']);
    Route::post('update', [SettingsController::class, 'updateFees']);
});

Route::group(['prefix' => 'boosted-shop'], function (){
    Route::get('index', [BoostedShopController::class, 'adminBoostedShopIndex']);
    Route::post('add', [BoostedShopController::class, 'addBoostedShop']);
    Route::post('edit', [BoostedShopController::class, 'editBoostedShop']);
    Route::post('delete', [BoostedShopController::class, 'deleteBoostedShop']);
    Route::get('filter', [BoostedShopController::class, 'adminBoostedShopFilter']);
});

Route::group(['prefix' => 'verification-requests'], function(){
    Route::get('/', [ProfileController::class, 'getVerificationRequests']);
    Route::post('update', [SettingsController::class, 'updateFees']);
});
